require ('levels')
local map 							-- stores tiledata
local mapWidth, mapHeight 			-- width and height in tiles
local mapX, mapY
local oldMapX, oldMapY				-- used in movement
local tilesWidthNum, tilesHeightNum -- number of tiles to show
local tilesetImage					-- Image of tiles
local tileQuads = {} 				-- parts of the tilesetImage used for different tiles
local tilesetSprite
local tileSize = 16
local tilesetBatch
windowWidth, windowHeight = love.window.getDimensions()
zoomX = ((windowWidth/20) - (windowWidth/20)%tileSize)/tileSize
zoomY = ((windowHeight/15) - (windowHeight/15)%tileSize)/tileSize

function mapLoad(level)				--Starts map functions
	map = level.tilemap				-- get the tilemap from the level
	mapHeight = table.getn(level.tilemap)
	mapWidth = table.getn(level.tilemap[1])
	mapX = level.startX				-- set where the player begins
	mapY = level.startY				-- set where the player begins
	tilesWidthNum = math.ceil(windowWidth/(tileSize*zoomX))
	tilesHeightNum = math.ceil(windowHeight/(tileSize*zoomY))
	tilesetImage = level.tileset	-- also, get the image we made the map with.
	tilesetImage:setFilter("nearest", "linear") 			-- this "linear filter" removes some artifacts if we were to scale the tiles
	if tilesetImage ~= nil then
		tileQuads = initQuads(tilesetImage);
		tilesetUpdate()
	end
end

function initQuads(im)				--Creates all Tiles + initiates Spritebatch
	local listQuads = {}			--creates a blank table
	local heightQuads = math.floor(im:getHeight()/tileSize)	--seperates the height of the screen into the height of the tiles
	local widthQuads = math.floor(im:getWidth()/tileSize)	--seperates the width of the screen into the width of the tiles
	local count = 0					--sets variable count to 0
	for j = 0, heightQuads do		-- for each y axis quad
		for i = 0, widthQuads do 	--for each x axis quad
			listQuads[count] = love.graphics.newQuad(i*tileSize,j*tileSize,tileSize,tileSize,im:getWidth(),im:getHeight())
			count = count + 1
		end
	end
	tilesetBatch = love.graphics.newSpriteBatch(tilesetImage, mapWidth * mapHeight)--makes a spritebatch
	return listQuads
end

function tilesetUpdate()			--Loads all tiles
	tilesetBatch:clear() 			--clears map
	local tilenum = 0
	for x=1, mapHeight do
		for y=1, mapWidth do
			tilesetBatch:add(tileQuads[map[y][x]],x*tileSize, y*tileSize)
			if map[x][y] == 2 then 	--triggered each time tile #2 is found being drawn
				levels.level1.collision.x[tilenum] = ((x-1)*tileSize)
				levels.level1.collision.y[tilenum] = ((y-1)*tileSize)
				tilenum = tilenum + 1
			end
		end
	end
	tilesetBatch:flush()
end

function moveMap(dx, dy)			--Moves map
	oldMapX = mapX
	oldMapY = mapY
	mapX = math.max(math.min(mapX + dx, (mapWidth + 1) - tilesWidthNum/2), (1 - tilesWidthNum/2))
	mapY = math.max(math.min(mapY + dy, (mapHeight + 1) - tilesHeightNum/2), (1 - tilesHeightNum/2))
	if math.floor(mapX) ~= math.floor(oldMapX) or math.floor(mapY) ~= math.floor(oldMapY) then
		tilesetUpdate()
	end
end

function mapDraw()					--Draws map
	love.graphics.draw(tilesetBatch, math.floor((-zoomX*(mapX)*tileSize)), math.floor(-zoomY*(mapY)*tileSize), 0, zoomX, zoomY)
end
